//
//  RNAMapLocation.m
//  RNAMapLocation
//
//  Created by Run on 2017/8/22.
//  Copyright © 2017年 Run. All rights reserved.
//

#import "RNAMapLocation.h"
#import <AMapLocationKit/AMapLocationKit.h>
#import <AMapFoundationKit/AMapFoundationKit.h>

#define DefaultLocationTimeout 10
#define DefaultReGeocodeTimeout 5

@interface RNAMapLocation ()<AMapLocationManagerDelegate>

@property (nonatomic, strong) AMapLocationManager *manager;

@end

@implementation RNAMapLocation

RCT_EXPORT_MODULE();

RCT_EXPORT_METHOD(singleLocation:(BOOL)reGeocode
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject){
    [self.manager setDelegate:self];
    [self.manager  setDesiredAccuracy:kCLLocationAccuracyHundredMeters];
    [self.manager setReGeocodeTimeout:DefaultReGeocodeTimeout];
    [self.manager setLocationTimeout:DefaultLocationTimeout];
    [self.manager setPausesLocationUpdatesAutomatically:NO];
    [self.manager requestLocationWithReGeocode:reGeocode completionBlock:^(CLLocation *location, AMapLocationReGeocode *regeocode, NSError *error) {
        if (error != nil && error.code == AMapLocationErrorLocateFailed)
        {
            //定位错误：此时location和regeocode没有返回值，不进行annotation的添加
            NSLog(@"定位错误:{%ld - %@};", (long)error.code, error.localizedDescription);
            NSString *code = [NSString stringWithFormat:@"%ld",error.code];
            reject(code,error.localizedDescription,error);
        }
        else if (error != nil
                 && (error.code == AMapLocationErrorReGeocodeFailed
                     || error.code == AMapLocationErrorTimeOut
                     || error.code == AMapLocationErrorCannotFindHost
                     || error.code == AMapLocationErrorBadURL
                     || error.code == AMapLocationErrorNotConnectedToInternet
                     || error.code == AMapLocationErrorCannotConnectToHost))
        {
            //逆地理错误：在带逆地理的单次定位中，逆地理过程可能发生错误，此时location有返回值，regeocode无返回值，进行annotation的添加
            //      NSLog(@"逆地理错误:{%ld - %@};", (long)error.code, error.localizedDescription);
            NSString *code = [NSString stringWithFormat:@"%ld",error.code];
            reject(code,error.localizedDescription,error);
            return;
        }
        else if (error != nil && error.code == AMapLocationErrorRiskOfFakeLocation)
        {
            //存在虚拟定位的风险：此时location和regeocode没有返回值，不进行annotation的添加
            //      NSLog(@"存在虚拟定位的风险:{%ld - %@};", (long)error.code, error.localizedDescription);
            NSString *code = [NSString stringWithFormat:@"%ld",error.code];
            reject(code,error.localizedDescription,error);
            
            return;
        }
        else
        {
            //没有错误：location有返回值，regeocode是否有返回值取决于是否进行逆地理操作，进行annotation的添加
        }
        
        //修改label显示内容
        if (regeocode)
        {
            //     ;
            //      NSLog(@"%@", [NSString stringWithFormat:@"%@ \n %@-%@-%.2fm", regeocode.formattedAddress,regeocode.citycode, regeocode.adcode, location.horizontalAccuracy]);
            resolve(@{@"address":regeocode.formattedAddress,
                      @"citycode":regeocode.citycode,
                      @"adcode":regeocode.adcode,
                      @"accuracy":@(location.horizontalAccuracy)});
        }
        else
        {
            //      NSLog(@"%@", [NSString stringWithFormat:@"lat:%f;lon:%f \n accuracy:%.2fm", location.coordinate.latitude, location.coordinate.longitude, location.horizontalAccuracy]);
            resolve(@{@"latitude":@(location.coordinate.latitude),
                      @"longitude":@(location.coordinate.longitude),
                      @"adcode":@"",
                      @"accuracy":@(location.horizontalAccuracy)});
        }
        
    }];
};

+ (void)initKey:(NSString *)key {
    if (!key) {
        NSString *reason = [NSString stringWithFormat:@"apiKey为空，请检查key是否正确设置。"];
        
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:reason delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
        
        [alert show];
    }
    [AMapServices sharedServices].apiKey = key;
}

- (AMapLocationManager *)manager {
    if (!_manager) {
        _manager = [[AMapLocationManager alloc] init];
    }
    return _manager;
}

@end
