//
//  RNAMapViewManager.m
//  FCSDemo
//
//  Created by Run on 2017/9/5.
//  Copyright © 2017年 Facebook. All rights reserved.
//

#import "RNAMapViewManager.h"
#import "RNMapView.h"
#import "RNMAAnnotationView.h"

#define POI_ID      @"poi"
#define CIRCLE_ID   @"circle"
#define DISTRICT_ID @"district"
#define POLYGON_ID  @"polygon"

@implementation RNAMapViewManager {
    RNMapView           *_amapView;
    AMapSearchAPI       *_search;
    AMapGeoFenceManager *_geoManager;
    NSString            *_customID;
}

RCT_EXPORT_MODULE(RNAMapView);
#pragma mark  -------  地理围栏管理者  -------
- (void)geoFenceManager {
    _geoManager = [[AMapGeoFenceManager alloc] init];
    _geoManager.delegate = self;
    _geoManager.activeAction = AMapGeoFenceActiveActionInside | AMapGeoFenceActiveActionOutside | AMapGeoFenceActiveActionStayed;
    _geoManager.allowsBackgroundLocationUpdates = YES;
}

#pragma mark  -------  设置围栏  -------
//POI
RCT_EXPORT_METHOD(setGeoFenceKeyword:(NSString *)keyword POIType:(NSString *)type city:(NSString *)city size:(NSInteger)size customID:(NSString *)ID) {
    //  [_geoManager addKeywordPOIRegionForMonitoringWithKeyword:@"北京大学" POIType:@"学府|车站" city:@"北京" size:15 customID:@"POI 1"];
    [_geoManager addKeywordPOIRegionForMonitoringWithKeyword:keyword POIType:type city:city size:size customID:ID];
}

//周边POI
RCT_EXPORT_METHOD(setGeoFenceAroundPOIRegionForMonitoringWithLocationPoint:(NSDictionary *)point aroundRadius:(NSInteger)radius keyword:(NSString *)keyword POIType:(NSString *)type size:(NSInteger)size customID:(NSString *)ID) {
    CLLocationCoordinate2D coor = [self getCoordinate2D:point];
    [_geoManager addAroundPOIRegionForMonitoringWithLocationPoint:coor aroundRadius:radius keyword:keyword POIType:type size:size customID:ID];
}

//Circle
RCT_EXPORT_METHOD(setGeoFenceCircleRegionForMonitoringWithCenter:(NSDictionary *)center radius:(CLLocationDistance)radius customID:(NSString *)ID) {
    CLLocationCoordinate2D coor = [self getCoordinate2D:center];
    [_geoManager addCircleRegionForMonitoringWithCenter:coor radius:radius customID:ID];
}

//行政区District
RCT_EXPORT_METHOD(setGeoFenceDistrictName:(NSString *)name customID:(NSString *)ID ) {
    [_geoManager addDistrictRegionForMonitoringWithDistrictName:name customID:ID];
}

//Polygon
RCT_EXPORT_METHOD(setGeoFencePolygonRegionForMonitoringWithCoordinates:(NSArray *)coordinates customID:(NSString *)ID ) {
    NSInteger index = coordinates.count;
    CLLocationCoordinate2D coors[index];
    for (NSInteger i = 0; i < coordinates.count; i ++) {
        CLLocationCoordinate2D coor = [self getCoordinate2D:coordinates[i]];
        coors[i] = coor;
    }
    
    [_geoManager addPolygonRegionForMonitoringWithCoordinates:coors count:index customID:ID];
}

- (CLLocationCoordinate2D)getCoordinate2D:(NSDictionary *)coordinate {
    CGFloat latitude = [[coordinate objectForKey:@"latitude"] floatValue];
    CGFloat longitude = [[coordinate objectForKey:@"longitude"] floatValue];
    CLLocationCoordinate2D coor;
    coor.longitude   =  longitude;
    coor.latitude    =  latitude;
    return coor;
}

#pragma mark  -------  AMapGeoFenceManagerDelegate  -------
- (void)amapGeoFenceManager:(AMapGeoFenceManager *)manager didAddRegionForMonitoringFinished:(NSArray <AMapGeoFenceRegion *> *)regions customID:(NSString *)customID error:(NSError *)error {
    if (error) {
        [self showAlert:@"提示" message:error.localizedDescription];
        return;
    }
    if ([customID containsString:POI_ID]) {
        for (AMapGeoFencePOIRegion * region in regions) {
            MACircle *circle = [MACircle circleWithCenterCoordinate:region.center radius:region.radius];
            [_amapView addOverlay:circle];
        }
        _amapView.centerCoordinate = [(AMapGeoFencePOIRegion *)regions.firstObject center];
        _amapView.zoomLevel = 13;
    }
    else if ([customID containsString:CIRCLE_ID]) {
        AMapGeoFenceCircleRegion *circleRegion = (AMapGeoFenceCircleRegion *)regions.firstObject;
        MACircle *circle = [MACircle circleWithCenterCoordinate:circleRegion.center radius:circleRegion.radius];
        [_amapView addOverlay:circle];
        [_amapView setVisibleMapRect:circle.boundingMapRect edgePadding:UIEdgeInsetsMake(20, 20, 20, 20) animated:YES];
        
    }
    else if ([customID containsString:POLYGON_ID]) {
        for (AMapGeoFencePolygonRegion *region in regions) {
            MAPolygon *polygonOverlay = [MAPolygon polygonWithCoordinates:region.coordinates count:region.count];
            [_amapView addOverlay:polygonOverlay];
            [_amapView setVisibleMapRect:polygonOverlay.boundingMapRect edgePadding:UIEdgeInsetsMake(20, 20, 20, 20) animated:YES];
        }
        
    }
    else if ([customID containsString:DISTRICT_ID]) {
        for (AMapGeoFenceDistrictRegion *region in regions) {
            for (NSArray *arealocation in region.polylinePoints) {
                
                CLLocationCoordinate2D *coorArr = malloc(sizeof(CLLocationCoordinate2D) * arealocation.count);
                
                for (int i = 0; i < arealocation.count; i++) {
                    AMapLocationPoint *point = [arealocation objectAtIndex:i];
                    coorArr[i] = CLLocationCoordinate2DMake(point.latitude, point.longitude);
                }
                
                MAPolygon *polygonOverlay = [MAPolygon polygonWithCoordinates:coorArr count:arealocation.count];
                [_amapView addOverlay:polygonOverlay];
                [_amapView setVisibleMapRect:polygonOverlay.boundingMapRect edgePadding:UIEdgeInsetsMake(20 ,20, 20, 20) animated:YES];
            }
        }
    }
    else {
        [self showAlert:@"提示" message:@"customID设置不正确"];
    }
}

- (void)showAlert:(NSString *)title message:(NSString *)message {
    UIAlertController *controller = [UIAlertController alertControllerWithTitle:title message:message preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *action = [UIAlertAction actionWithTitle:@"确定" style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
        
    }];
    [controller addAction:action];
    [[self sh_viewController] presentViewController:controller animated:YES completion:nil];
}

- (UIViewController *)sh_viewController
{
    UIResponder *responder = _amapView;
    while ((responder = [responder nextResponder])){
        if ([responder isKindOfClass: [UIViewController class]]){
            return (UIViewController *)responder;
        }
    }
    return nil;
}



- (void)amapGeoFenceManager:(AMapGeoFenceManager *)manager didGeoFencesStatusChangedForRegion:(AMapGeoFenceRegion *)region customID:(NSString *)customID error:(NSError *)error {
    /**
     AMapGeoFenceRegionStatusInside   = 1,       ///< 在范围内
     AMapGeoFenceRegionStatusOutside  = 2,       ///< 在范围外
     AMapGeoFenceRegionStatusStayed   = 3,
     */
    
    
}

#pragma mark  -------  移除围栏  -------
RCT_EXPORT_METHOD(doClean) {
    [_amapView removeOverlays:_amapView.overlays];
    [_geoManager removeAllGeoFenceRegions];
}


- (UIView *)view {
    RNMapView *view = [[RNMapView alloc] initWithFrame:[UIScreen mainScreen].bounds];
    view.delegate = self;
    _search = [[AMapSearchAPI alloc] init];
    _search.delegate = self;
    _amapView = view;
    [self geoFenceManager];
    
    //  CLLocationCoordinate2D coordinates[4];
    //  coordinates[0].latitude = 39.781892;
    //  coordinates[0].longitude = 116.293413;
    //
    //  coordinates[1].latitude = 39.787600;
    //  coordinates[1].longitude = 116.391842;
    //
    //  coordinates[2].latitude = 39.733187;
    //  coordinates[2].longitude = 116.417932;
    //
    //  coordinates[3].latitude = 39.704653;
    //  coordinates[3].longitude = 116.338255;
    //  MAPolygon *polygon = [MAPolygon polygonWithCoordinates:coordinates count:4];
    //  [amapView addOverlay:polygon];
    return view;
}

RCT_EXPORT_VIEW_PROPERTY(onChange, RCTBubblingEventBlock)
RCT_EXPORT_VIEW_PROPERTY(mapType, int)
RCT_EXPORT_VIEW_PROPERTY(showTraffic, BOOL)
RCT_EXPORT_VIEW_PROPERTY(spot, NSDictionary*)
RCT_EXPORT_VIEW_PROPERTY(markStr, NSString)
RCT_EXPORT_VIEW_PROPERTY(strokeColor, UIColor)
RCT_EXPORT_VIEW_PROPERTY(fillColor, UIColor)
RCT_EXPORT_VIEW_PROPERTY(lineWidth, double)
RCT_EXPORT_VIEW_PROPERTY(alpha, double)


- (NSDictionary *)constantsToExport {
    return @{
             @"UserTrackingMode":@{
                     @"none":@(MAUserTrackingModeNone),
                     @"follow":@(MAUserTrackingModeFollow),
                     @"heading":@(MAUserTrackingModeFollowWithHeading),
                     },
             @"type":@{
                     @"Standard":@(MAMapTypeStandard),
                     @"Satellite":@(MAMapTypeSatellite),
                     }
             };
}

RCT_EXPORT_METHOD(setCenterCoordinate:(nonnull NSDictionary *)coordinate Options:(nonnull NSNumber *)reactTag ){
    dispatch_async(self.bridge.uiManager.methodQueue, ^{
        [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *,UIView *> *viewRegistry) {
            id view = viewRegistry[reactTag];
            NSLog(@"View- -- -- -- -- %@",view);
            RNMapView *newView = (RNMapView *)view;
            [amapView removeAnnotations:amapView.annotations];
            CGFloat latitude = [[coordinate objectForKey:@"latitude"] floatValue];
            CGFloat longitude = [[coordinate objectForKey:@"longitude"] floatValue];
            NSString *title = [coordinate objectForKey:@"title"] ;
            CLLocationCoordinate2D coor = {latitude,longitude};
            MAPointAnnotation *ani = [[MAPointAnnotation alloc] init];
            ani.coordinate = coor;
            ani.title = title;
            [newView addAnnotation:ani];
            [newView setCenterCoordinate:CLLocationCoordinate2DMake(latitude, longitude) animated:YES];
            //      [amapView addAnnotation:ani];
            //      [amapView setCenterCoordinate:CLLocationCoordinate2DMake(latitude, longitude) animated:YES];
        }];
    });
}

RCT_EXPORT_METHOD(setOverlay:(NSArray *)coors) {
    if (coors) {
        NSInteger index = [coors count];
        CLLocationCoordinate2D coordinates[index];
        
        NSMutableArray *array = [NSMutableArray array];
        for (NSInteger i = 0; i < index; i ++) {
            NSDictionary *coor = coors[i];
            CGFloat latitude = [[coor objectForKey:@"latitude"] floatValue];
            CGFloat longitude = [[coor objectForKey:@"longitude"] floatValue];
            coordinates[i].latitude = latitude;
            coordinates[i].longitude = longitude;
        }
        
        MAPolygon *polygon = [MAPolygon polygonWithCoordinates:coordinates count:index];
        [_amapView addOverlay:polygon];
    }
}


- (void)mapView:(MAMapView *)mapView didSelectAnnotationView:(MAAnnotationView *)view {
    RNMapView *newMap = (RNMapView *)mapView;
    newMap.onChange( @{
                       @"latitude": @([[view annotation] coordinate].latitude),
                       @"longitude": @([[view annotation] coordinate].longitude)
                       });
    
}

- (MAAnnotationView *)mapView:(MAMapView *)mapView viewForAnnotation:(id<MAAnnotation>)annotation {
    
    if ([annotation isKindOfClass:[MAPointAnnotation class]])
    {
        static NSString *pointReuseIndetifier = @"pointReuseIndetifier";
        
        MAPinAnnotationView *annotationView = (MAPinAnnotationView *)[mapView dequeueReusableAnnotationViewWithIdentifier:pointReuseIndetifier];
        if (annotationView == nil)
        {
            annotationView = [[MAPinAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:pointReuseIndetifier];
        }
        
        annotationView.canShowCallout   = YES;
        annotationView.animatesDrop     = YES;
        annotationView.draggable        = NO;
        if ([(RNMapView *)mapView markStr]) {
            annotationView.image            = [UIImage imageNamed:[(RNMapView *)mapView markStr]];
        }
        else {
            annotationView.pinColor = MAPinAnnotationColorPurple;
        }
        
        return annotationView;
        
        //    static NSString *reuseIndetifier = @"annotationReuseIndetifier";
        //    RNMAAnnotationView *annotationView = (RNMAAnnotationView *)[mapView dequeueReusableAnnotationViewWithIdentifier:reuseIndetifier];
        //    if (annotationView == nil)
        //    {
        //      annotationView = [[RNMAAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:reuseIndetifier];
        //    }
        //    annotationView.image = [UIImage imageNamed:@"home_bg_new"];
        //
        //    // 设置为NO，用以调用自定义的calloutView
        //    annotationView.canShowCallout = NO;
        //
        //    // 设置中心点偏移，使得标注底部中间点成为经纬度对应点
        //    annotationView.centerOffset = CGPointMake(0, -18);
        //
        //    return annotationView;
    }
    
    
    return nil;
}

- (MAOverlayRenderer *)mapView:(MAMapView *)mapView rendererForOverlay:(id<MAOverlay>)overlay {
    if ([overlay isKindOfClass:[MAPolygon class]])
    {
        MAPolygonRenderer *polygonRenderer = [[MAPolygonRenderer alloc] initWithPolygon:overlay];
        polygonRenderer.lineWidth   = 4.f;
        polygonRenderer.strokeColor = _amapView.strokeColor? _amapView.strokeColor:[UIColor colorWithRed:0 green:1 blue:0 alpha:1];
        polygonRenderer.fillColor   = [UIColor redColor];
        polygonRenderer.alpha       = 0.4;
        return polygonRenderer;
    }
    if ([overlay isKindOfClass:[MAPolyline class]])
    {
        MAPolylineRenderer *polylineRenderer = [[MAPolylineRenderer alloc] initWithPolyline:overlay];
        
        polylineRenderer.lineWidth    = 8.f;
        polylineRenderer.strokeColor  = [UIColor colorWithRed:0 green:1 blue:0 alpha:0.6];
        polylineRenderer.lineJoin = kCGLineJoinRound;
        polylineRenderer.lineCap  = kCGLineCapRound;
        
        return polylineRenderer;
    }
    if ([overlay isKindOfClass:[MACircle class]]) {
        MACircleRenderer *circleRenderer = [[MACircleRenderer alloc] initWithCircle:overlay];
        
        circleRenderer.lineWidth    = 5.f;
        circleRenderer.strokeColor  = [UIColor colorWithRed:0.6 green:0.6 blue:0.6 alpha:0.8];
        circleRenderer.fillColor    = [UIColor colorWithRed:1.0 green:0.8 blue:0.0 alpha:0.8];
        circleRenderer.alpha        = 0.4;
        return circleRenderer;
    }
    
    
    
    return nil;
}


RCT_EXPORT_METHOD(mapsearch:(NSString *)keywords :(NSString *)types) {
    
    AMapPOIKeywordsSearchRequest *request = [[AMapPOIKeywordsSearchRequest alloc] init];
    request.keywords = keywords;
    request.types = types;
    
    [_search AMapPOIKeywordsSearch:request];
    
}

//search delegate
- (void)onPOISearchDone:(AMapPOISearchBaseRequest *)request response:(AMapPOISearchResponse *)response {
    
    if (response.pois.count == 0)
    {
        return;
    }
    NSMutableArray *resultList = [NSMutableArray array];
    NSLog(@"%@",response.pois);
    NSMutableArray *annotations = [NSMutableArray array];
    
    [_amapView removeAnnotations:amapView.annotations];
    [response.pois enumerateObjectsUsingBlock:^(AMapPOI * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        MAPointAnnotation *annotation = [[MAPointAnnotation alloc] init];
        [resultList addObject:@{
                                @"uid": obj.uid,
                                @"name": obj.name,
                                @"type": obj.type,
                                @"typecode": obj.typecode,
                                @"latitude": @(obj.location.latitude),
                                @"longitude": @(obj.location.longitude),
                                @"address": obj.address,
                                @"tel": obj.tel,
                                @"distance": @(obj.distance)
                                }];
        CLLocationCoordinate2D coor = {obj.location.latitude,obj.location.longitude};
        annotation.coordinate = coor;
        annotation.title = obj.name;
        [annotations addObject:annotation];
    }];
    
    //[self.bridge.eventDispatcher sendDeviceEventWithName:@"name" body:resultList];
    //
    CustomEventEmitter *emitter = [[CustomEventEmitter alloc] init];
    emitter.bridge = self.bridge;
    [emitter addListener:@"name"];
    [emitter sendEventWithName:@"name" body:resultList];
    
    [_amapView addAnnotations:annotations];
    [_amapView setCenterCoordinate:[(MAPointAnnotation *)annotations[0] coordinate] animated:YES];
    
    
    
}

- (void)AMapSearchRequest:(id)request didFailWithError:(NSError *)error
{
    //    NSLog(@"Error: %@", error);
    NSLog(@"POI error -------- %@",error);
}

+ (void)initKey:(NSString *)key {
    if (!key) {
        NSString *reason = [NSString stringWithFormat:@"apiKey为空，请检查key是否正确设置。"];
        
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:reason delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
        
        [alert show];
    }
    [AMapServices sharedServices].apiKey = key;
}



@end

@implementation CustomEventEmitter

- (NSArray<NSString *> *)supportedEvents {
    return @[@"name"];
}

@end
